from typing import Optional, Tuple, TYPE_CHECKING

from PyQt5 import uic
from PyQt5.QtCore import QCoreApplication, pyqtSignal
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QGroupBox, QLineEdit
from PyQt5.QtWidgets import QWidget, QDialog, QPushButton, QMessageBox

from astra_installer.settings.manager import SettingsManager
from astra_installer.settings.systeminfo import AstraEdition
from astra_installer_qt.utils import Utils as QtUtils

if TYPE_CHECKING:
    from gettext import gettext as _

translate = QCoreApplication.translate

dialog_ui = QtUtils.file_path('grub', 'ui', 'grub_settings_dialog.ui')
UiFormClass, UiFormBase = uic.loadUiType(dialog_ui)  # type: QDialog, QDialog


class GrubSettingsDialog(UiFormClass, UiFormBase):
    settings_changed = pyqtSignal(bool, str, str)

    def __init__(self, parent: QWidget = None) -> None:
        super().__init__(parent=parent)
        self._grub_password_group_box = None  # type: Optional[QGroupBox]
        self._grub_password_line_edit = None  # type: Optional[QLineEdit]
        self._grub_password_confirmation_line_edit = None  # type: Optional[QLineEdit]
        self._apply_push_button = None # type: Optional[QPushButton]
        self._cancel_push_button = None # type: Optional[QPushButton]
        self.setupUi(self)
        self.__setup()

    def __setup(self) -> None:
        self.setModal(True)
        self.adjustSize()
        self._apply_push_button.clicked.connect(self.accept)
        self._cancel_push_button.clicked.connect(self.reject)
        self._grub_password_group_box.setChecked(SettingsManager.systeminfo.edition == AstraEdition.SE)

    def __is_valid_password(self) -> Tuple[bool, str]:
        if not self._grub_password_group_box.isChecked():
            return True, str()

        password = self._grub_password_line_edit.text()
        password_confirmation = self._grub_password_confirmation_line_edit.text()

        if not password and not password_confirmation:
            return True, str()
        if password != password_confirmation:
            return False, translate("text", "GRUB passwords don't match")
        if len(password) < 8:
            return False, translate("text", "GRUB password cannot be shorter than 8 characters")

        return True, str()

    def show(self) -> None:
        active_window = QCoreApplication.instance().activeWindow()
        self.setVisible(True)
        self.exec()
        if active_window:
            active_window.activateWindow()

    def retranslate(self, __: str) -> None:
        self.retranslateUi(self)

    def accept(self) -> None:
        self.settings_changed.emit(self._grub_password_group_box.isChecked(),
                                   self._grub_password_line_edit.text(),
                                   self._grub_password_confirmation_line_edit.text())

        valid, error_message = self.__is_valid_password()
        if valid:
            return super().accept()
        else:
            dialog = QMessageBox(parent=self)
            dialog.setIcon(QMessageBox.Critical)
            dialog.setWindowTitle(translate("text", "GRUB settings error"))
            dialog.setText(error_message)
            dialog.setStandardButtons(QMessageBox.Close | QMessageBox.Ignore)
            dialog.button(QMessageBox.Close).setText(translate('text', 'Return'))
            dialog.button(QMessageBox.Ignore).setText(translate('text', 'Force quit'))
            dialog.button(QMessageBox.Close).setFocus()

            exit_code = dialog.exec()
            self.activateWindow()
            if exit_code == QMessageBox.Ignore:
                return self.reject()

    def reject(self) -> None:
        self._grub_password_group_box.setChecked(SettingsManager.systeminfo.edition == AstraEdition.SE)
        self._grub_password_line_edit.setText(str())
        self._grub_password_confirmation_line_edit.setText(str())
        self.settings_changed.emit(self._grub_password_group_box.isChecked(),
                                   self._grub_password_line_edit.text(),
                                   self._grub_password_confirmation_line_edit.text())
        return super().reject()

    def closeEvent(self, event: QCloseEvent) -> None:
        event.ignore()
        self.reject()
