from time import strftime, gmtime

from PyQt5.QtCore import QObject, pyqtSignal, QThread

from astra_installer.logs.logger import Logger
from astra_installer.settings.manager import SettingsManager
from astra_installer.settings.progress_manager import ProgressManager, InstallationStatus
from astra_installer.setup.setup_creator import SetupCreator
from astra_installer_qt.log_manager import LogManager
from astra_installer_qt.progress_manager import ProgressManager as UIProgressManager


class SetupThread(QThread):
    max_changed = pyqtSignal(int)
    progress_changed = pyqtSignal(int, str)
    install_finished = pyqtSignal(InstallationStatus)
    log_appended = pyqtSignal(str)

    def __init__(self) -> None:
        super().__init__()
        self.setup = SetupCreator.create()
        ProgressManager.max_callback = self.on_backend_max_changed
        ProgressManager.progress_callback = self.on_backend_progress_changed
        ProgressManager.finished_callback = self.on_backend_finished
        Logger.log_callback = self.on_backend_log_appended

    def on_backend_max_changed(self, value: int) -> None:
        self.max_changed.emit(value)

    def on_backend_progress_changed(self, value: int, description: str) -> None:
        self.progress_changed.emit(value, description)

    def on_backend_finished(self, status: InstallationStatus) -> None:
        self.install_finished.emit(status)

    def on_backend_log_appended(self, message: str) -> None:
        self.log_appended.emit(message)

    def run(self) -> None:
        self.setup.setup()


class InstallationManager(QObject):
    finished = pyqtSignal()

    def __init__(self, progress: UIProgressManager, log: LogManager) -> None:
        super(InstallationManager, self).__init__()
        self.progress = progress
        self.log = log
        self.setup = SetupThread()
        self.setup.max_changed.connect(self.progress.progressbar.setMaximum)
        self.setup.progress_changed.connect(self.progress.set_progress)
        self.setup.install_finished.connect(self.on_backend_finished)
        self.setup.log_appended.connect(self.log.log)

    def install(self) -> None:
        self.progress.progressbar.setMaximum(ProgressManager.max())
        ProgressManager.start_timer()
        self.setup.start()

    def on_backend_finished(self, status: InstallationStatus) -> None:
        self.progress.progressbar.setValue(self.progress.progressbar.maximum())
        self.progress.hide()
        if status.error:
            self.log.log(self.tr('Installation completed with error.'))
            self.log.open_installation_log()
        elif SettingsManager.systeminfo.oem:
            msgs = []
            if status.failed_steps:
                msgs.append(self.tr('Failed to setup: {steps}.').format(steps=', '.join(status.failed_steps)))
                msgs.append(self.tr('See setup logs for more info.'))
            time = strftime('%H:%M:%S', gmtime(ProgressManager.get_past_time().total_seconds()))
            self.log.open_installation_info(time, '<br>'.join(msgs))
        else:
            time = strftime('%H:%M:%S', gmtime(ProgressManager.get_past_time().total_seconds()))
            self.log.open_installation_info(time)
        self.finished.emit()
