import gi
gi.require_version("Gtk", "3.0")
gi.require_version("Gdk", "3.0")

from gi.repository import Gtk

from blivetgui.config import config
from blivetgui.dialogs.helpers import supported_raids, get_monitor_size
from blivetgui.dialogs.widgets import RaidChooser
from blivetgui.dialogs.add_dialog import AddDialog
from blivet import size

from astra_installer_qt.storage.blivet.installer_blivet_widgets import InstallerBlivetEncryptionChooser

class InstallerBlivetAddDialog(AddDialog):
    def __init__(self, parent_window, selected_parent, selected_free, available_free,
                 supported_filesystems, mountpoints=None, installer_mode=False):
        """

            :param str parent_type: type of (future) parent device
            :param parent_device: future parent device
            :type parent_device: :class:`blivet.Device` instances
            :param free_device: selected free space device
            :type free_device: :class:`blivetgui.utils.FreeSpaceDevice` instances
            :param list free_pvs: list PVs with no VG
            :param free_disks_regions: list of free regions on non-empty disks
            :type free_disks_regions: list of :class:`blivetgui.utils.FreeSpaceDevice` instances
            :param list mountpoints: list of mountpoints in current devicetree
            :param bool installer_mode: installer mode

          """

        self.parent_window = parent_window

        self.selected_parent = selected_parent
        self.selected_free = selected_free

        self.available_free = available_free

        self.installer_mode = installer_mode
        self.mountpoints = mountpoints

        self.supported_filesystems = supported_filesystems
        self.supported_raids = supported_raids()

        Gtk.Dialog.__init__(self)

        self.set_title(_("Create new device"))
        self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                         Gtk.STOCK_OK, Gtk.ResponseType.OK)
        self.set_transient_for(self.parent_window)

        if self.installer_mode:
            self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
            self.set_decorated(False)

        self.set_resizable(False)  # auto shrink after removing widgets

        mwidth, mheight = get_monitor_size(self.parent_window)
        self.max_height = int(mheight * 0.80)  # take at most 80 % of current monitor height
        self.max_width = int(mwidth * 0.80)  # take at most 80 % of current monitor width

        self.grid = Gtk.Grid(column_homogeneous=False, row_spacing=10, column_spacing=5)
        self.grid.set_border_width(10)

        self.scrolledwindow = Gtk.ScrolledWindow()
        self.scrolledwindow.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.NEVER)

        self.box = self.get_content_area()
        self.box.add(self.scrolledwindow)

        self.scrolledwindow.add(self.grid)

        self.widgets_dict = {}

        self.filesystems_store, self.filesystems_combo = self.add_fs_chooser()
        self.label_entry, self.name_entry = self.add_name_chooser()
        self.parents_store = self.add_parent_list()

        # encryption chooser
        self._encryption_chooser = InstallerBlivetEncryptionChooser()
        self._encryption_chooser.connect("encrypt-toggled", self.on_encrypt_check)
        self.grid.attach(self._encryption_chooser.grid, 0, 12, 3, 1)
        self.widgets_dict["encrypt"] = [self._encryption_chooser]

        self._encryption_chooser._visible_widgets[0].set_text(_("Use protective transformation:"))
        self._encryption_chooser._advanced_widgests[0].set_text(_("Type of protective transformation:"))

        # raid chooser
        self._raid_chooser = RaidChooser()
        self._raid_chooser.connect("changed", self.on_raid_type_changed)
        self.grid.attach(self._raid_chooser.box, 0, 5, 3, 1)
        self.widgets_dict["raid"] = [self._raid_chooser]

        if self.installer_mode:
            self.mountpoint_entry = self.add_mountpoint()

        self.devices_combo = self.add_device_chooser()
        self.devices_combo.connect("changed", self.on_devices_combo_changed)

        self.size_area = None
        self.add_size_area()

        self.advanced = None

        self.md_type_combo = self.add_md_type_chooser()

        # adjust size/scrolling of the dialog with every change of its size
        self._resize_handler = self.connect("configure-event", self.scrolled_adjust)

        self.show_all()

        self.devices_combo.set_active(0)

        ok_button = self.get_widget_for_response(Gtk.ResponseType.OK)
        ok_button.connect("clicked", self.on_ok_clicked)

    def on_filesystems_combo_changed(self, _combo):
        super().on_filesystems_combo_changed(_combo)

        if self.selected_fs:
            if self.selected_fs.type == "biosboot":
                units = self.size_area.main_chooser.available_units
                for index, unit in enumerate(units):
                    self.size_area.main_chooser._unit_chooser.set_active(index)
                    if size.unit_str(unit) == "MiB":
                        break
                self._encryption_chooser.encrypt = False
                self.hide_widgets([ "encrypt", "size" ])
            else:
                units_count = len(self.size_area.main_chooser.available_units)
                index = (units_count - 1) if units_count > 0 else 0
                self.size_area.main_chooser._unit_chooser.set_active(index)
                self.show_widgets([ "encrypt", "size" ])

    def on_devices_combo_changed(self, _event):
        if (self.selected_type == "btrfs volume") \
            or (self.selected_type == "btrfs subvolume"):
            if self.selected_fs:
                if self.selected_fs.type == "biosboot":
                    if config.default_fstype in (fs.type for fs in self.supported_filesystems):
                        self.filesystems_combo.set_active_id(config.default_fstype)

        super().on_devices_combo_changed(_event)
