from typing import Optional, TYPE_CHECKING
from locale import setlocale, LC_MESSAGES
from os import environ

from PyQt5.QtCore import QObject, QCoreApplication
from PyQt5.QtWidgets import QPushButton, QLabel, QMainWindow
from pyudev import Device

from astra_installer.settings.manager import SettingsManager
from astra_installer.storage.storage import Storage
from astra_installer.storage.template.storage_template import StorageTemplate, TemplateType
from astra_installer.storage.utils.pyudev_utils import device_size
from astra_installer_qt.storage.ui.storage_settings_dialog import StorageSettingsDialog

translate = QCoreApplication.translate

if TYPE_CHECKING:
    from gettext import gettext as _


class StorageManager(QObject):
    def __init__(self, main_window: QMainWindow, settings_button: QPushButton, info_label: QLabel) -> None:
        super().__init__()
        self.__settings_button = settings_button
        self.__info_label = info_label
        self.__storage = SettingsManager.storage
        self.__settings_dialog = StorageSettingsDialog(storage=self.__storage, parent=main_window)
        self.__setup()

    @property
    def storage(self) -> Storage:
        return self.__storage

    @property
    def settings_dialog(self) -> StorageSettingsDialog:
        return self.__settings_dialog

    def retranslate(self, locale: str) -> None:
        try:
            environ['LANG'] = '{locale}.UTF-8'.format(locale=locale)
            setlocale(LC_MESSAGES, '{locale}.UTF-8'.format(locale=locale))
        except Exception:
            print('Failed to set locale to {}'.format(locale))

        self.__settings_dialog.retranslate(locale)
        self.__update_info_label()

    def __setup(self) -> None:
        self.__settings_button.clicked.connect(self.__settings_button_clicked)
        self.__storage.current_template_changed.connect(self.__current_template_changed)
        self.__settings_dialog.load()

    def __settings_button_clicked(self) -> None:
        self.__settings_dialog.show()

    def __current_template_changed(self, template: StorageTemplate) -> None:
        if not template:
            return

        template.usable_device_changed.connect(self.__template_usable_device_changed)
        self.__update_info_label()

    def __template_usable_device_changed(self, _: Optional[Device]) -> None:
        self.__update_info_label()

    def __update_info_label(self) -> None:
        if not self.__storage.current_template:
            text = _('Storage layout unset')
        elif self.__storage.current_template.usable_device:
            text = _(self.__storage.current_template.name)
            if self.__storage.current_template.snapshots_support:
                if self.__storage.current_template.recomended:
                    text += " ({}, {})".format(_('recomended'), _('snapshots are supported'))
                else:
                    text += " ({})".format(_('snapshots are supported'))
            elif self.__storage.current_template.recomended:
                text += " ({})".format(_('recomended'))
            count_bytes = device_size(self.__storage.current_template.usable_device)
            gb_size = round(count_bytes / 1024 / 1024 / 1024, 2)

            device_info = '{size} {gb_text}'.format(size=gb_size, gb_text=translate('text', 'Gb'))
            model = self.__storage.current_template.usable_device.properties.get('ID_MODEL')
            if model:
                device_info += ' {model}'.format(model=model)
            device_info += ' ' + self.__storage.current_template.usable_device.device_node

            text += ' - ' + device_info
        else:
            if self.__storage.current_template.type == TemplateType.Custom:
                text = _(self.__storage.current_template.name)
            else:
                text = _('Storage device not found')

        self.__info_label.setText(text)
