import datetime
from typing import Optional, TYPE_CHECKING

import pytz
from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QComboBox
from babel.dates import get_timezone_location

from astra_installer.settings.manager import SettingsManager
from astra_installer.translator import deferredtr as _tr

if TYPE_CHECKING:
    from gettext import gettext as _


class TimeZoneManager(QObject):
    babel_unknown_regions = {'GMT': 'GMT',
                             'UTC': 'UTC',
                             'Europe/Kirov': _tr('Russia (Kirov) Time'),
                             'Europe/Astrakhan': _tr('Russia (Astrakhan) Time'),
                             'Europe/Ulyanovsk': _tr('Russia (Ulyanovsk) Time'),
                             'Asia/Yangon': _tr('Myanmar (Yangon) Time'),
                             'Asia/Barnaul': _tr('Russia (Barnaul) Time'),
                             'Asia/Tomsk': _tr('Russia (Tomsk) Time'),
                             'America/Fort_Nelson': _tr('United Kingdom (Fort Nelson) Time')
                             }

    def __init__(self, combobox: QComboBox) -> None:
        super(TimeZoneManager, self).__init__()
        self.combobox = combobox
        self.backend = SettingsManager.timezone
        self.selected = None  # type: Optional[str]
        self.locale = None  # type: Optional[str]
        self.setup()

    def setup(self) -> None:
        self.refresh_timezones()
        self.combobox.activated.connect(self.on_combobox_activated)

    def create_timezone_items(self, timezones, locale=None) -> list:
        items = []
        for timezone in timezones:
            time_now = datetime.datetime.now(timezone)
            time_now_str = time_now.strftime('(%H:%M %d.%m.%Y)')
            offset = time_now.strftime('%z')
            offset_str = '{}:{}'.format(offset[:-2], offset[-2:])
            # Workaround for BT-9832
            timezone_str = timezone.__str__()
            if timezone_str in self.babel_unknown_regions:
                timezone_str = _(self.babel_unknown_regions.get(timezone_str, timezone_str))
            else:
                timezone_str = get_timezone_location(timezone, locale if locale is not None else 'en')
            item_str = 'UTC{} {} {}'.format(offset_str, timezone_str, time_now_str)
            items.append((item_str, timezone.__str__()))
        return items

    def refresh_timezones(self, locale=None) -> None:
        timezones = self.backend.available_timezones(locale)
        timezones.sort(key=lambda timezone: datetime.datetime.now(timezone).strftime('%z'))

        self.combobox.clear()
        for text, timezone in self.create_timezone_items(timezones, locale):
            self.combobox.addItem(text, timezone)

        default_timezone = self.backend.default_timezone(locale)
        if default_timezone is not None:
            self.selected = default_timezone.__str__()
        index = self.combobox.findData(self.selected)
        self.combobox.setCurrentIndex(index if index >= 0 else 0)
        if SettingsManager.systeminfo.arch == "e2k-8c":
            self.combobox.model().sort(1)

        self.on_combobox_activated()

    def refresh_timezones_text(self) -> None:
        timezones = []
        for item in range(self.combobox.count()):
            timezones.append(pytz.timezone(self.combobox.itemData(item)))

        current_index = self.combobox.currentIndex()
        self.combobox.clear()
        for text, timezone in self.create_timezone_items(timezones, self.locale):
            self.combobox.addItem(text, timezone)
        self.combobox.setCurrentIndex(current_index)

    def on_combobox_activated(self) -> None:
        self.selected = self.combobox.currentData()
        self.setup_backend()

    def setup_backend(self) -> None:
        self.backend.selected = self.combobox.currentData()

    def retranslate(self, locale) -> None:
        self.locale = locale[:2].lower()
        if SettingsManager.systeminfo.oem:
            self.refresh_timezones_text()
        else:
            self.refresh_timezones(self.locale)
